package cn.com.qmth.stmms.api.controller.admin;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import cn.com.qmth.stmms.biz.exam.bean.ProblemHistoryVo;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import com.qmth.boot.core.collection.PageResult;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.admin.dto.ProblemHistoryDTO;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.model.ProblemHistory;
import cn.com.qmth.stmms.biz.mark.model.ProblemType;
import cn.com.qmth.stmms.biz.mark.query.ProblemHistorySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.mark.service.ProblemHistoryService;
import cn.com.qmth.stmms.biz.mark.service.ProblemTypeService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.HistoryStatus;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.utils.DateUtils;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "问题试卷")
@Controller("adminProblemHistoryController")
@RequestMapping("/api/admin/exam/problem/history")
public class ProblemHistoryController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(ProblemHistoryController.class);

    public static HistoryStatus[] STATUS_LIST = { HistoryStatus.WAITING, HistoryStatus.BACK };

    private static final String DEFAULT_SECRET_NUMBER = "***";
    @Autowired
    private ProblemTypeService problemService;

    @Autowired
    private ProblemHistoryService historyService;
    private ExamService examService;

    @Autowired
    private MarkGroupService groupService;
    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private UserService userService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private LockService lockService;

    @Autowired
    private MarkService markService;

    @ApiOperation(value = "问题卷列表")
    @Logging(menu = "查询问题卷", type = LogType.QUERY)
    @RequestMapping(value = "query", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ProblemHistoryDTO> query(ProblemHistorySearchQuery query) {
        ApiUser user = getApiUser();
        int examId = getSessionExamId();
        Exam exam = examService.findById(examId);
        List<ProblemType> problemTypes = problemService.findByExamId(examId);
        Map<Integer, ProblemType> problemMap = new HashMap<Integer, ProblemType>();
        for (ProblemType problemType : problemTypes) {
            problemMap.put(problemType.getId(), problemType);
        }
        if (user.isSubjectHeader() && StringUtils.isBlank(query.getSubjectCode())) {
            String subjectCodeIn = StringUtils.join(user.getSubjectCodeSet(), ",");
            query.setSubjectCodeIn(subjectCodeIn);
        }
        if (query.getStatus() == null) {
            query.setStatus(HistoryStatus.WAITING);
        }
        query.setExamId(examId);
        query.orderByExamNumber();
        query = historyService.findByQuery(query);
        List<ProblemHistoryDTO> list = new LinkedList<ProblemHistoryDTO>();
        for (ProblemHistory history : query.getResult()) {
            ExamStudent student = studentService.findById(history.getStudentId());
            ProblemHistoryDTO dto = new ProblemHistoryDTO();
            dto.setSubjectCode(student.getSubjectCode());
            dto.setSubjectName(student.getSubjectName());
            dto.setStatus(query.getStatus().getName());
            dto.setGroupNumber(history.getGroupNumber());
            if(user.isSubjectHeader() && exam.isForbiddenInfo()){
                dto.setExamNumber(DEFAULT_SECRET_NUMBER);
                dto.setSecretNumber(DEFAULT_SECRET_NUMBER);
                dto.setName(DEFAULT_SECRET_NUMBER);
            }else{
                dto.setExamNumber(student.getExamNumber());
                dto.setSecretNumber(student.getSecretNumber());
                dto.setName(student.getName());
            }
            dto.setProblemType(problemMap.get(history.getProblemId()).getName());
            if (history.getMarkerId() != null) {
                User u = userService.findByMarkerId(history.getMarkerId());
                dto.setMarkLogin(u.getLoginName() + " " + u.getName());
                dto.setMarkTime(DateUtils.formatDateTime(history.getCreateTime()));
            }
            if (history.getUserId() != null) {
                User u = userService.findById(history.getUserId());
                dto.setProblemUserName(u.getLoginName() + " " + u.getName());
                dto.setProblemRestTime(DateUtils.formatDateTime(history.getUpdateTime()));
            }
            list.add(dto);
        }
        return PageUtil.of(list, query);
    }

    @ApiOperation(value = "问题卷重置")
    @Logging(menu = "问题卷重置", type = LogType.UPDATE)
    @RequestMapping(value = "/restBatch", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage save(@RequestParam Integer[] libraryIds) {
        ApiUser wu = RequestUtils.getApiUser();
        for (Integer libraryId : libraryIds) {
            MarkLibrary library = libraryService.findById(libraryId);
            if (library != null && subjectCheck(library.getSubjectCode(), wu)) {
                try {
                    lockService.watch(LockType.EXAM_SUBJECT, library.getExamId(), library.getSubjectCode());
                    lockService.watch(LockType.GROUP, library.getExamId(), library.getSubjectCode(),
                            library.getGroupNumber());
                    lockService.waitlock(LockType.STUDENT, library.getStudentId());
                    if (library.getStatus().equals(LibraryStatus.PROBLEM)) {
                        markService.rejectLibrary(library, wu.getUser().getId(), null, true);
                    }
                } catch (Exception e) {
                    log.error("back library error", e);
                } finally {
                    lockService.unlock(LockType.STUDENT, library.getStudentId());
                    lockService.unwatch(LockType.GROUP, library.getExamId(), library.getSubjectCode(),
                            library.getGroupNumber());
                    lockService.unwatch(LockType.EXAM_SUBJECT, library.getExamId(), library.getSubjectCode());
                }
            }
        }
        return resultOk();
    }

    @ApiOperation(value = "导出问题卷")
    @Logging(menu = "导出问题卷", type = LogType.EXPORT)
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    @ResponseBody
    public void exportFile(ProblemHistorySearchQuery query,HttpServletResponse response) {
        try {
            String fileName = "问题卷数据.xlsx";
            ApiUser user = getApiUser();
            int examId = getSessionExamId();
            Exam exam = examService.findById(examId);
            List<ProblemType> problemTypes = problemService.findByExamId(examId);
            Map<Integer, ProblemType> problemMap = new HashMap<Integer, ProblemType>();
            for (ProblemType problemType : problemTypes) {
                problemMap.put(problemType.getId(), problemType);
            }
            if (user.isSubjectHeader() && StringUtils.isBlank(query.getSubjectCode())) {
                String subjectCodeIn = StringUtils.join(user.getSubjectCodeSet(), ",");
                query.setSubjectCodeIn(subjectCodeIn);
            }
            if (query.getStatus() == null) {
                query.setStatus(HistoryStatus.WAITING);
            }
            query.setExamId(examId);
            query.setPageNumber(1);
            query.setPageSize(Integer.MAX_VALUE);
            query.orderByExamNumber();
            query = historyService.findByQuery(query);
            List<ProblemHistoryDTO> list = new LinkedList<ProblemHistoryDTO>();
            for (ProblemHistory history : query.getResult()) {
                ExamStudent student = studentService.findById(history.getStudentId());
                ProblemHistoryDTO dto = new ProblemHistoryDTO();
                dto.setSubjectCode(student.getSubjectCode());
                dto.setSubjectName(student.getSubjectName());
                dto.setStatus(query.getStatus().getName());
                dto.setGroupNumber(history.getGroupNumber());
                if(user.isSubjectHeader() && exam.isForbiddenInfo()){
                    dto.setExamNumber(DEFAULT_SECRET_NUMBER);
                    dto.setSecretNumber(DEFAULT_SECRET_NUMBER);
                    dto.setName(DEFAULT_SECRET_NUMBER);
                }else{
                    dto.setExamNumber(student.getExamNumber());
                    dto.setSecretNumber(student.getSecretNumber());
                    dto.setName(student.getName());
                }
                dto.setProblemType(problemMap.get(history.getProblemId()).getName());
                if (history.getMarkerId() != null) {
                    User u = userService.findByMarkerId(history.getMarkerId());
                    dto.setMarkLogin(u.getLoginName() + " " + u.getName());
                    dto.setMarkTime(DateUtils.formatDateTime(history.getCreateTime()));
                }
                if (history.getUserId() != null) {
                    User u = userService.findById(history.getUserId());
                    dto.setProblemUserName(u.getLoginName() + " " + u.getName());
                    dto.setProblemRestTime(DateUtils.formatDateTime(history.getUpdateTime()));
                }
                list.add(dto);
            }
            new ExportExcel("问题卷数据", ProblemHistoryDTO.class).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导出问题卷数据失败！"+e.getMessage());
        }
    }

}
