package cn.com.qmth.stmms.api.controller.admin;

import java.math.BigDecimal;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.admin.vo.ScanInfoVO;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.model.ExamPackage;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.query.ExamPackageSearchQuery;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamPackageService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.query.ExamSubjectSearchQuery;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.utils.ExportExcel;

@Api(tags = "扫描管理")
@Controller("adminScanController")
@RequestMapping("/api/admin/scan")
public class ScanController extends BaseApiController {

    protected static final Logger log = LoggerFactory.getLogger(ScanController.class);

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamPackageService packageService;

    private FileService fileService;

    @ApiOperation(value = "扫描进度-按科目查询")
    @RequestMapping(value = "/subject", method = RequestMethod.POST)
    @Logging(menu = "扫描进度-按科目查询", type = LogType.QUERY)
    @ResponseBody
    public PageResult<ScanInfoVO> subjectList(HttpServletRequest request, ExamSubjectSearchQuery query) {
        int examId = getSessionExamId(request);
        List<ScanInfoVO> list = new LinkedList<>();
        query.setExamId(examId);
        query = subjectService.findByQuery(query);
        for (ExamSubject subject : query.getResult()) {
            ScanInfoVO vo = new ScanInfoVO();
            vo.setName(subject.getCode() + "-" + subject.getName()
                    + (StringUtils.isNotBlank(subject.getRemark())
                            ? ("-" + StringUtils.trimToEmpty(subject.getRemark()))
                            : ""));
            vo.setTotalCount(studentService.countByExamIdAndSubjectCode(examId, subject.getCode()));
            vo.setScanCount(subject.getUploadCount());
            long sheetCount = studentService.countSheetCountByExamIdAndSubjectCode(examId, subject.getCode());
            long sheetCount2 = new BigDecimal(sheetCount).divide(new BigDecimal(2), 0, BigDecimal.ROUND_HALF_UP)
                    .longValue();
            vo.setScanSheetCount(sheetCount2);
            ExamStudentSearchQuery studentSearchQuery = new ExamStudentSearchQuery();
            studentSearchQuery.setExamId(examId);
            studentSearchQuery.setSubjectCode(subject.getCode());
            studentSearchQuery.setManualAbsent(true);
            long manualAbsentCount = studentService.countByQuery(studentSearchQuery);
            vo.setManualAbsentCount(manualAbsentCount);
            list.add(vo);
        }
        PageResult<ScanInfoVO> pageResult = new PageResult<>();
        pageResult.setResult(list);
        pageResult.setPageNumber(query.getPageNumber());
        pageResult.setPageSize(query.getPageSize());
        pageResult.setTotalCount(query.getTotalCount());
        pageResult.setPageCount(query.getCurrentCount());
        return pageResult;
    }

    @ApiOperation(value = "扫描进度-按科目导出")
    @Logging(menu = "扫描进度-按科目导出", type = LogType.EXPORT)
    @RequestMapping(value = "/subject/export", method = RequestMethod.POST)
    public void export(ExamSubjectSearchQuery query, HttpServletRequest request, HttpServletResponse response) {
        int examId = getSessionExamId(request);
        List<ScanInfoVO> list = new LinkedList<>();
        query.setExamId(examId);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        query = subjectService.findByQuery(query);
        for (ExamSubject subject : query.getResult()) {
            ScanInfoVO vo = new ScanInfoVO();
            vo.setName(subject.getCode() + "-" + subject.getName()
                    + (StringUtils.isNotBlank(subject.getRemark())
                            ? ("-" + StringUtils.trimToEmpty(subject.getRemark()))
                            : ""));
            vo.setTotalCount(studentService.countByExamIdAndSubjectCode(examId, subject.getCode()));
            vo.setScanCount(studentService.countByExamIdAndSubjectCode(examId, subject.getCode(), true));
            vo.setScanSheetCount(studentService.countSheetCountByExamIdAndSubjectCode(examId, subject.getCode()) / 2);
            ExamStudentSearchQuery studentSearchQuery = new ExamStudentSearchQuery();
            studentSearchQuery.setExamId(examId);
            studentSearchQuery.setSubjectCode(subject.getCode());
            studentSearchQuery.setManualAbsent(true);
            long manualAbsentCount = studentService.countByQuery(studentSearchQuery);
            vo.setManualAbsentCount(manualAbsentCount);
            list.add(vo);
        }
        try {
            String fileName = "扫描进度.xlsx";
            new ExportExcel("扫描进度", ScanInfoVO.class).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("scan subject export error!", e);
            throw new StatusException("导出扫描进度失败！" + e.getMessage());
        }
    }

    @ApiOperation(value = "扫描进度-按签到表查询")
    @RequestMapping(value = "/package", method = RequestMethod.POST)
    @Logging(menu = "扫描进度-按签到表查询", type = LogType.QUERY)
    @ResponseBody
    public PageResult<ExamPackage> packageList(HttpServletRequest request, ExamPackageSearchQuery query) {
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query = packageService.findByQuery(query);
        for (ExamPackage ep : query.getResult()) {
            ep.setUrls(fileService.getPackageUris(ep.getExamId(), ep.getCode(), 1, ep.getPicCount()));
        }
        return PageUtil.of(query);
    }

    @ApiOperation(value = "扫描进度-按考点查询")
    @Logging(menu = "扫描进度-按考点查询", type = LogType.QUERY)
    @RequestMapping(value="/examSite", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ScanInfoVO> examSite(HttpServletRequest request, ExamStudentSearchQuery query) {
        int examId = getSessionExamId(request);
        List<ScanInfoVO> list = new LinkedList<>();
        List<String> result = studentService.findDistinctExamSite(examId);
        result = result.stream().filter(s -> StringUtils.isNotBlank(s)).collect(Collectors.toList());
        if (StringUtils.isBlank(query.getExamSite())) {
            List<?> examSiteList = cn.com.qmth.stmms.admin.utils.PageUtil.startPage(result, query.getPageNumber(),
                    query.getPageSize());
            if (examSiteList != null) {
                for (Object e : examSiteList) {
                    ScanInfoVO vo = new ScanInfoVO();
                    String examSite = (String) e;
                    vo.setName(examSite);
                    vo.setTotalCount(studentService.countByExamIdAndExamSite(examId, examSite));
                    vo.setScanCount(studentService.countByExamIdAndExamSite(examId, examSite, true));
                    vo.setScanSheetCount(studentService.countSheetCountByExamIdAndExamSite(examId, examSite) / 2);
                    ExamStudentSearchQuery studentSearchQuery = new ExamStudentSearchQuery();
                    studentSearchQuery.setExamId(examId);
                    studentSearchQuery.setExamSite(examSite);
                    studentSearchQuery.setManualAbsent(true);
                    long manualAbsentCount = studentService.countByQuery(studentSearchQuery);
                    vo.setManualAbsentCount(manualAbsentCount);
                    list.add(vo);
                }
                query.setTotalCount(result.size());
                query.setTotalPage(cn.com.qmth.stmms.admin.utils.PageUtil.pageCount(result, query.getPageNumber(),
                        query.getPageSize()));
                query.setCurrentCount(examSiteList.size());
            }
        } else {
            ScanInfoVO vo = new ScanInfoVO();
            vo.setName(query.getExamSite());
            vo.setTotalCount(studentService.countByExamIdAndExamSite(examId, query.getExamSite()));
            vo.setScanCount(studentService.countByExamIdAndExamSite(examId, query.getExamSite(), true));
            vo.setScanSheetCount(studentService.countSheetCountByExamIdAndExamSite(examId, query.getExamSite()) / 2);
            ExamStudentSearchQuery studentSearchQuery = new ExamStudentSearchQuery();
            studentSearchQuery.setExamId(examId);
            studentSearchQuery.setExamSite(query.getExamSite());
            studentSearchQuery.setManualAbsent(true);
            long manualAbsentCount = studentService.countByQuery(studentSearchQuery);
            vo.setManualAbsentCount(manualAbsentCount);
            list.add(vo);
            query.setTotalCount(1);
            query.setTotalPage(1);
            query.setCurrentCount(1);
        }
        PageResult<ScanInfoVO> pageResult = new PageResult<>();
        pageResult.setResult(list);
        pageResult.setPageNumber(query.getPageNumber());
        pageResult.setPageSize(query.getPageSize());
        pageResult.setTotalCount(query.getTotalCount());
        pageResult.setPageCount(query.getCurrentCount());
        return pageResult;
    }

    @ApiOperation(value = "扫描进度-按考点导出")
    @Logging(menu = "扫描进度-按考点导出", type = LogType.EXPORT)
    @RequestMapping(value = "/examSite/export", method = RequestMethod.POST)
    public void examSiteExport(ExamStudentSearchQuery query, HttpServletRequest request, HttpServletResponse response) {
        int examId = getSessionExamId(request);
        List<ScanInfoVO> list = new LinkedList<>();
        List<String> result = studentService.findDistinctExamSite(examId);
        result = result.stream().filter(s -> StringUtils.isNotBlank(s)).collect(Collectors.toList());
        if (StringUtils.isBlank(query.getExamSite())) {
            for (String examSite : result) {
                ScanInfoVO vo = new ScanInfoVO();
                vo.setName(examSite);
                vo.setTotalCount(studentService.countByExamIdAndExamSite(examId, examSite));
                vo.setScanCount(studentService.countByExamIdAndExamSite(examId, examSite, true));
                vo.setScanSheetCount(studentService.countSheetCountByExamIdAndExamSite(examId, examSite) / 2);
                ExamStudentSearchQuery studentSearchQuery = new ExamStudentSearchQuery();
                studentSearchQuery.setExamId(examId);
                studentSearchQuery.setExamSite(examSite);
                studentSearchQuery.setManualAbsent(true);
                long manualAbsentCount = studentService.countByQuery(studentSearchQuery);
                vo.setManualAbsentCount(manualAbsentCount);
                list.add(vo);
            }
        } else {
            ScanInfoVO vo = new ScanInfoVO();
            vo.setName(query.getExamSite());
            vo.setTotalCount(studentService.countByExamIdAndExamSite(examId, query.getExamSite()));
            vo.setScanCount(studentService.countByExamIdAndExamSite(examId, query.getExamSite(), true));
            vo.setScanSheetCount(studentService.countSheetCountByExamIdAndExamSite(examId, query.getExamSite()) / 2);
            ExamStudentSearchQuery studentSearchQuery = new ExamStudentSearchQuery();
            studentSearchQuery.setExamId(examId);
            studentSearchQuery.setExamSite(query.getExamSite());
            studentSearchQuery.setManualAbsent(true);
            long manualAbsentCount = studentService.countByQuery(studentSearchQuery);
            vo.setManualAbsentCount(manualAbsentCount);
            list.add(vo);
        }
        try {
            String fileName = "扫描进度.xlsx";
            new ExportExcel("扫描进度", ScanInfoVO.class).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("scan subject export error!", e);
            throw new StatusException("导出扫描进度失败！" + e.getMessage());
        }
    }
}
