package cn.com.qmth.stmms.api.controller.admin;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.tools.io.ZipWriter;

import cn.com.qmth.stmms.admin.dto.ExamStudentDTO;
import cn.com.qmth.stmms.admin.utils.ExportStudentExcel;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.bean.ScoreVo;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamPackage;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamPackageService;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import com.qmth.boot.core.exception.StatusException;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.utils.Encodes;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "成绩查询")
@Controller("adminScoreController")
@RequestMapping("/api/admin/exam/score")
public class ScoreController extends BaseApiController {

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private ExamPackageService packageService;

    @Autowired
    private ExamService examService;

    @Autowired
    private FileService fileService;

    public static final String UN_SELECTIVE_SCORE = "-1";

    @Logging(menu = "成绩查询", type = LogType.QUERY)
    @ApiOperation(value = "分页查询")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ScoreVo> list(ExamStudentSearchQuery query) {
        Integer filter = query.getFilter();
        ApiUser wu = getApiUser();
        int examId = getSessionExamId();
        query.setExamId(examId);
        if (filter == 1) {
            query.setObjectiveScore(0d);
            query.setSubjectiveScoreGt(0d);
        } else if (filter == 2) {
            query.setObjectiveScoreGt(0d);
            query.setSubjectiveScore(0d);
        }
        if ((wu.isSubjectHeader() || wu.isSchoolViewer()) && StringUtils.isBlank(query.getSubjectCode())) {
            query.setSubjectCodes(new ArrayList<>(wu.getSubjectCodeSet()));
        }
        query.setRole(wu.getRole());
        query.setUserId(wu.getId());
        query = studentService.findByQueryPlus(query);
        for (ExamStudent student : query.getResult()) {
            buildSheetUrl(student);
            buildPackageUrl(student);
            buildAnswerUrl(student);
        }
        String logMessage = getLogMessage(query, filter);
        RequestUtils.setLog(logMessage);
        Exam exam = examService.findById(examId);
        List<ScoreVo> ret = new ArrayList<>();
        for (ExamStudent es : query.getResult()) {
            ScoreVo vo = ScoreVo.of(es);
            if ((wu.isSubjectHeader() || wu.isSchoolViewer()) && exam.isForbiddenInfo()) {
                vo.setExamNumber("***");
                vo.setName("***");
                vo.setStudentCode("***");
            }
            ret.add(vo);
        }
        return PageUtil.of(ret, query);
    }

    private String getLogMessage(ExamStudentSearchQuery query, Integer filter) {
        StringBuilder builder = new StringBuilder();
        if (StringUtils.isNotBlank(query.getName())) {
            builder.append("姓名:").append(query.getName()).append(";");
        }
        if (StringUtils.isNotBlank(query.getExamNumber())) {
            builder.append("准考证号:").append(query.getExamNumber()).append(";");
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            builder.append("学号:").append(query.getStudentCode()).append(";");
        }
        if (StringUtils.isNotBlank(query.getSubjectCode())) {
            builder.append("科目代码:").append(query.getSubjectCode()).append(";");
        }
        if (filter == 1) {
            builder.append("客观0分，主观有分").append(";");
        }
        if (filter == 2) {
            builder.append("客观有分，主观0分").append(";");
        }
        if (query.getAbsent() != null) {
            builder.append("缺考:").append(query.getAbsent() ? "缺考" : "正常").append(";");
        }
        if (query.getUpload() != null) {
            builder.append("上传:").append(query.getUpload() ? "已上传" : "未上传").append(";");
        }
        if (query.getBreach() != null) {
            builder.append("缺考:").append(query.getBreach() ? "违纪" : "正常").append(";");
        }
        if (StringUtils.isNotBlank(query.getExamSite())) {
            builder.append("考点:").append(query.getExamSite()).append(";");
        }
        if (StringUtils.isNotBlank(query.getExamRoom())) {
            builder.append("考场:").append(query.getExamRoom()).append(";");
        }
        if (StringUtils.isNotBlank(query.getCollege())) {
            builder.append("学院:").append(query.getCollege()).append(";");
        }
        if (StringUtils.isNotBlank(query.getClassName())) {
            builder.append("班级:").append(query.getClassName()).append(";");
        }
        if (StringUtils.isNotBlank(query.getTeacher())) {
            builder.append("任课老师:").append(query.getTeacher()).append(";");
        }
        if (query.getStartScore() != null) {
            builder.append("总分从").append(query.getStartScore()).append("到").append(query.getEndScore()).append(";");
        }
        return builder.toString();
    }

    @Logging(menu = "导出成绩", type = LogType.EXPORT)
    @ApiOperation(value = "导出成绩")
    @RequestMapping(value = "export", method = RequestMethod.POST)
    @ResponseBody
    public void export(ExamStudentSearchQuery query, HttpServletResponse response) {
        ApiUser wu = getApiUser();
        Exam exam = examService.findById(getSessionExamId());
        if (exam == null || !exam.getSchoolId().equals(wu.getUser().getSchoolId())) {
            throw new StatusException("请选择正确的考试");
        }
        String fileName = "成绩单.xlsx";
        query.setExamId(exam.getId());
        Set<String> subjectSet = new HashSet<>();
        List<String> error = new ArrayList<String>();
        // 单科目导出
        if (StringUtils.isNotBlank(query.getSubjectCode())) {
            String exportMessage = subjectService.enableExport(exam, query.getSubjectCode());
            // 判断不允许导出则直接返回
            if (exportMessage != null) {
                throw new StatusException("该科目不能导出成绩 " + exportMessage);
            }
            subjectSet.add(query.getSubjectCode());
        }
        // 全体科目导出
        else {
            List<ExamSubject> list = subjectService.list(exam.getId());
            if (wu.isSubjectHeader() || wu.isSchoolViewer()) {
                list = getExamSubject(exam.getId(), wu);
            }
            for (ExamSubject subject : list) {
                // 遍历所有科目，判断是否允许导出
                String meaages = subjectService.enableExport(exam, subject.getCode());
                if (meaages == null) {
                    subjectSet.add(subject.getCode());
                } else {
                    error.add(meaages);
                }
            }
            if (list.size() != subjectSet.size()) {
                fileName = "成绩单(未导出全量数据).xlsx";
            }
        }
        try {
            List<ExamStudentDTO> list = new LinkedList<>();
            // 可导出科目不为空时，才能执行查询操作
            if (!subjectSet.isEmpty()) {
                query.setSubjectCodes(new ArrayList<>(subjectSet));
                query.setPageNumber(1);
                query.setPageSize(Integer.MAX_VALUE);
                query.setRole(wu.getRole());
                query.setUserId(wu.getId());
                query = studentService.findByQueryPlus(query);
                for (ExamStudent student : query.getResult()) {
                    if (!student.isUpload() || student.isBreach() || student.isAbsent()) {
                        student.setObjectiveScore(0d);
                        student.setSubjectiveScore(0d);
                    }
                    if (student.getSubjectiveScoreList() != null) {
                        student.setSubjectiveScoreList(
                                student.getSubjectiveScoreList().replace(UN_SELECTIVE_SCORE, "/"));
                    }
                    list.add(new ExamStudentDTO(student));
                }
            }
            if (StringUtils.isNotBlank(query.getSubjectCode())) {
                List<String> headerList = getOptionHeader(exam.getId(), query.getSubjectCode());
                ExportStudentExcel excel = new ExportStudentExcel("成绩单", headerList, ExamStudentDTO.class);
                excel.setDataList(list, true);
                excel.write(response, fileName).dispose();
            } else {
                response.setHeader("Content-Disposition", "attachment; filename=" + Encodes.urlEncode("成绩单.zip"));
                ZipWriter writer = ZipWriter.create(response.getOutputStream());
                ByteArrayOutputStream os = new ByteArrayOutputStream();
                new ExportExcel("成绩单", ExamStudentDTO.class).setDataList(list).write(os);
                byte[] value = os.toByteArray();
                writer.write(new ByteArrayInputStream(value), fileName);
                if (!error.isEmpty()) {
                    writer.write(StringUtils.join(error, ";").getBytes(), "error.txt");
                }
                writer.close();
            }
        } catch (Exception e) {
            throw new StatusException("导出成绩失败！" + e.getMessage());
        }
    }

    private List<String> getOptionHeader(int examId, String subjectCode) {
        List<String> headerList = new ArrayList<String>();
        List<ExamQuestion> oQuestions = questionService.findByExamAndSubjectAndObjectiveGroupByNumber(examId,
                subjectCode, true);
        List<ExamQuestion> sQestions = questionService.findByExamAndSubjectAndObjective(examId, subjectCode, false);
        for (ExamQuestion examQuestion : oQuestions) {
            headerList.add(getTitle(examQuestion) + "选项");
            headerList.add(getTitle(examQuestion) + "得分");
        }
        for (ExamQuestion examQuestion : sQestions) {
            headerList.add(getTitle(examQuestion));
        }
        return headerList;
    }

    private String getTitle(ExamQuestion examQuestion) {
        return examQuestion.getMainTitle() + " " + examQuestion.getMainNumber() + "-" + examQuestion.getSubNumber();
    }

    private void buildSheetUrl(ExamStudent student) {
        // 改为内部原图浏览地址，直接附带标记内容
        student.setSheetUrls(
                fileService.getSheetUris(student.getExamId(), student.getExamNumber(), 1, student.getSheetCount()));
    }

    private void buildPackageUrl(ExamStudent student) {
        if (StringUtils.isNotBlank(student.getPackageCode())) {
            ExamPackage ep = packageService.find(student.getExamId(), student.getPackageCode());
            if (ep != null && ep.getPicCount() > 0) {
                student.setPackageUrls(fileService.getPackageUris(ep.getExamId(), ep.getCode(), 1, ep.getPicCount()));
            }
        }
    }

    private void buildAnswerUrl(ExamStudent student) {
        ExamSubject subject = subjectService.find(student.getExamId(), student.getSubjectCode());
        if (subject != null && subject.getAnswerFileType() != null) {
            student.setAnswerUrl(
                    fileService.getAnswerUri(subject.getExamId(), subject.getCode(), subject.getAnswerFileType()));
        }
        if (subject != null) {
            student.setSubjectRemark(StringUtils.trimToEmpty(subject.getRemark()));
        }
    }

}
