package cn.com.qmth.stmms.common.support;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.api.exception.ApiException;

@ControllerAdvice
public class CustomExceptionHandler {

    protected static final Logger log = LoggerFactory.getLogger(BaseApiController.class);

    private static final String ERROR_MESSAGE_HEADER_KEY = "error-info";

    @ExceptionHandler(Exception.class)
    public ResponseEntity<StatusResponse> handleException(Exception e, HttpServletRequest request) {
        StatusResponse body = new StatusResponse(500, "系统异常");
        return asResult(e, body, request);
    }

    @ExceptionHandler(RuntimeException.class)
    public ResponseEntity<StatusResponse> handleException(RuntimeException e, HttpServletRequest request,
            HttpServletResponse response) {
        StatusResponse body = null;

        if (e instanceof StatusException) {
            StatusException se = (StatusException) e;
            body = new StatusResponse(se.getCode() == null ? 500 : se.getCode(), se.getMessage());
        } else if (e instanceof ApiException) {
            response.addHeader(ERROR_MESSAGE_HEADER_KEY, StringUtils.trimToEmpty(e.getMessage()));
            ApiException teme = (ApiException) e;
            body = new StatusResponse(teme.getCode(), teme.getMessage());
            return asResultApiEx(e, body, request);
        } else {
            log.error(e.getMessage(), e);
            body = new StatusResponse(500, "系统异常");
        }

        return asResult(e, body, request);
    }

    private ResponseEntity<StatusResponse> asResultApiEx(Throwable err, StatusResponse body,
            HttpServletRequest request) {
        ApiException teme = (ApiException) err;
        HttpStatus httpStatus = HttpStatus.valueOf(teme.getCode());

        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type", "application/json;charset=utf-8");
        return new ResponseEntity<>(body, headers, httpStatus);
    }

    private ResponseEntity<StatusResponse> asResult(Throwable err, StatusResponse body, HttpServletRequest request) {

        HttpStatus httpStatus = HttpStatus.INTERNAL_SERVER_ERROR;

        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type", "application/json;charset=utf-8");
        return new ResponseEntity<>(body, headers, httpStatus);
    }

}